var fs = require('fs'), 
	path = require('path');

function JasmineRunner(options, callback){

	var doServer = options.serve,
		port = options.port || 8080,
		pollFrequency = options.poll || 1000,
		runners = options.runners || null,
		that = this;

	this.isRunningTests = false;
	this.isTestRunQueued = false;
	this.reporter = this._setupReporter(); // async

	if(doServer){
		this.server = that._setupServer({
						port:port, 
						reporter: this.reporter
					});
	};

	if(runners){
		this.tests = this._setupRunners({
						runners: runners, 
						jasmineReporter: this.reporter.getJasmineReporter(),
					});
	} else {
		console.error("I don't have any tests to run!");
		process.exit(1);
	}

};

JasmineRunner.prototype._setupReporter = function(callback){
	console.debug("Setting up reporter");

	var reporter = require('./reporter-agregator.js').create({
			onDone: function(){ 
					console.debug("Reporter created.");
			}
	});

	return reporter;

};

JasmineRunner.prototype._setupServer = function(options){
	console.debug("Setting up server");

	var server = require('./server.js');
	server.start(options.port, function(){
		callback(server);
	});

	options.reporter.onUpdate(function(report){
		server.updateHtml(report.html);
		server.updateJson(report.simple);
	});

	return server;
};

JasmineRunner.prototype._setupRunners = function(options,callback){
	console.debug("Setting up tests.");

	var tests = [];
	var runners = options.runners || [],
		jasmineReporter = options.jasmineReporter;

	for(var k in runners){
		var runner = runners[k];
		console.debug('Creating test based on ' + runner);

		var test = require('./runner-from-html.js').create({
			runner: runner,
			jasmineReporter: jasmineReporter
		});
		tests.push(test);
	}

	console.debug("Finished creating tests ( based on " +tests.length + " runner/s).");

	return tests;
};

JasmineRunner.prototype.runTests = function(callback){

	var that = this;

	// We have to maintain synchronisity, otherwise things
	// get hectic.
	if(this.isRunningTests) {
		// DANGER: if more than one call to runTests is made when
		// tests are running, only the last callback will ever be
		// triggered.
		this.isTestRunQueued = true;
		this.queuedCallback = callback;
		console.debug("Cannot run suites concurrently. Queued another run.");
		return;
	};
	var whenAllTestsHaveRun = function(){
		console.debug("Finished running test suites.");

		if(callback) callback(that.reporter.getReport());
		that.reporter.reset();

		that.isRunningTests = false;
		if(that.isTestRunQueued){
			that.isTestRunQueued = false;
			that.runTests(that.queuedCallback);
			that.queuedCallback = false;
		}
	};

	console.debug("Running test suites.");
	this.isRunningTests = true;

	var queue = [],
		tests = this.tests;
	for(var i = 0; i < tests.length; i++) queue.push(tests[i]);

	// !! Is recursive
	var runNextTest = function(){
		if(queue.length == 0){
			whenAllTestsHaveRun();
			return;
		} 

		var test = queue.pop();
		console.debug("Running tests: " + test.name);
		test.run(function(){
			console.debug("Finished running suite: " + test.name);
			runNextTest();
		});
	};

	runNextTest();
};

exports.run = function(options){
	var onDone = options.onDone || function(){},
		debug = options.debug;

	console.debug = debug ? function(msg){ console.log(msg); } : function(msg){};

	var runner = new JasmineRunner(options);
	runner.runTests(onDone);
	return runner;
};